#include <dht22.h>
#include <stdio.h>
#include <stdint.h>


//    FILE: dhtdemo.c
//  AUTHOR: Greg LaPolla
// PURPOSE: DHT22 library test 

// DHT PIN layout from left to right
// =================================
// FRONT : DESCRIPTION
// pin 1 : VCC
// pin 2 : DATA
// pin 3 : Not Connected
// pin 4 : GND

init(5);   // ESP 16    UNO 5    MKR1010 5

void main()
{
  printf("dhtdemo.c");
  printf("LIBRARY VERSION: ");
  println(DHTNEW_LIB_VERSION);
  println();

  println("\n1. Type detection test, first run might take longer to determine type");
  println("STAT\tHUMI\tTEMP\tTIME\tTYPE");
  test();
  test();
  test();
  test();

  println("\n2. Humidity offset test");
  println("STAT\tHUMI\tTEMP\tTIME\tTYPE");
  setHumOffset(2.5);
  test();
  setHumOffset(5.5);
  test();
  setHumOffset(-5.5);
  test();
  setHumOffset(0);
  test();

  println("\n3. Temperature offset test");
  println("STAT\tHUMI\tTEMP\tTIME\tTYPE");
  setTempOffset(2.5);
  test();
  setTempOffset(5.5);
  test();
  setTempOffset(-5.5);
  test();
  setTempOffset(0);
  test();

  println("\n4. LastRead test");
  read();
  for (int i = 0; i < 20; i++)
  {
    if (millis() - mySensor.lastRead() > 1000)
    {
      mySensor.read();
      println("actual read");
    }
    print(mySensor.getHumidity(), 1);
    print(",\t");
    println(mySensor.getTemperature(), 1);
    delay(250);
  }

  println("\nDone...");
}


void test()
{
  // READ DATA
  uint32_t start = micros();
  int chk = mySensor.read();
  uint32_t stop = micros();

  switch (chk)
  {
    case DHTLIB_OK:
      print("OK,\t");
      break;
    case DHTLIB_ERROR_CHECKSUM:
      print("Checksum error,\t");
      break;
    case DHTLIB_ERROR_TIMEOUT_A:
      print("Time out A error,\t");
      break;
    case DHTLIB_ERROR_TIMEOUT_B:
      print("Time out B error,\t");
      break;
    case DHTLIB_ERROR_TIMEOUT_C:
      print("Time out C error,\t");
      break;
    case DHTLIB_ERROR_TIMEOUT_D:
      print("Time out D error,\t");
      break;
    case DHTLIB_ERROR_SENSOR_NOT_READY:
      print("Sensor not ready,\t");
      break;
    case DHTLIB_ERROR_BIT_SHIFT:
      print("Bit shift error,\t");
      break;
    case DHTLIB_WAITING_FOR_READ:
      print("Waiting for read,\t");
      break;
    default:
      print("Unknown: ");
      print(chk);
      print(",\t");
      break;
  }

  // DISPLAY DATA
  print(mySensor.getHumidity(), 1);
  print(",\t");
  print(mySensor.getTemperature(), 1);
  print(",\t");
  uint32_t duration = stop - start;
  print(duration);
  print(",\t");
  println(mySensor.getType());
  delay(500);
}
// -- END OF FILE --
